module.exports = [
  {
    name: '-h, --help',
    description: 'Display help',
  },
  {
    name: '-q, --quiet',
    description: 'Set output log level to quiet',
  },
  {
    name: '-v, --verbose',
    description: 'Set output log level to verbose',
  },
  {
    name: '-d, --debug',
    description: 'Set output log level to debug',
  },
  {
    name: '--json',
    description: 'Enable JSON output',
  },
]
