let deno = () => `export async function handler (event: object) {
  console.log(JSON.stringify(event, null, 2))
  return
}
`

let node = () => `export async function handler (event) {
  console.log(JSON.stringify(event, null, 2))
  return
}
`

let ruby = () => `def handler(event)
  puts event
  true
end
`

let python = () => `def handler(event, context):
  print(event)
  return True
`

module.exports = { node, deno, ruby, python }
