module.exports = function () {
  return `
<enhance-page-container>
  <main>
    <h1 class="mb1 font-semibold text3">Email Verified</h1>
    <p>You are logged in. You can close this window and return to your other page.</p>
</main>
</enhance-page-container>
`
}
