module.exports = function () {
  return `// For production allow list add users below where keys match unique property of OAuth user
export default {
  appAccounts: {
    // janedoe: {
    //   role: 'member',
    //   name: 'Jane Doe'
    // }
  }
}`
}
